/***************************************************************************

                     F O X B O R O   I / A   S e r i e s

            P L B   B L O C K   S O U R C E   D O C U M E N T E R

                               by Kevin Millar

 Rev 1.0 - Jun 1990 - Initial writing ( written for I/A Version 2.1 )
 Rev 1.1 - Nov 1993 - Updated for I/A Version 3.2 (AP50 - SunOS)
 Rev 1.2 - May 1999 - Tidied for release to public domain (I/A Version 6.0)

***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <varargs.h>

#include "plb_dump.h"

int form_feed();
int print_dump();
int print_labels();
int print_ladder();
int print_timers();
int print_xref();
int print_hex(char *encrypt, char *dest);
int pprintf();
int print_centre(char *string);
int load_source(char *cmpnd,char *block);

void print_title();
void print_summary();
void print_help();


main(argc, argv)
int argc;
char *argv[];
{
   char c;
   int i;
   unsigned int j, id;
   unsigned char row, col, c_type;
   char string[80];
   char cmpnd[32], block[32];

   /* initialise compound and block names */
   cmpnd[0] = '\0';
   block[0] = '\0';

   /* process command line */
   if (argc<2)
   {
      print_title();
      print_summary();
      exit(0);
   }

   /* loop thru arguments on command line */
   for (i=1;i<argc;i++)
   {
      if ( argv[i][0]=='-' )
      {
		 /* process arguments that start with a "-" */
         for (j=1; j<strlen(argv[i]); j++)
         {
            c = tolower(argv[i][j]);
            switch (c)
            {
               case 'l':
                  label_flag = TRUE;
                  j++;
                  if (argv[i][j]=='a' || argv[i][j]=='A')
                     all_label_flag = TRUE;
                  else
                     j--;
                  break;
               case 'r':
                  logic_flag = TRUE;
                  break;
               case 'x':
                  xref_flag = TRUE;
                  break;
               case 'h':
                  if ( strcmp(argv[i],"-hex")==0 || strcmp(argv[i],"-HEX")==0 )
                    dump_flag = TRUE;
                  else
                    help_flag = TRUE;
                  j = strlen( argv[i] );
                  break;
               case 't':
                  timer_flag = TRUE;
                  j++;
                  if (argv[i][j]=='a' || argv[i][j]=='A')
                     all_timer_flag = TRUE;
                  else
                     j--;
                  break;
               case 'p':
                  j++;
                  if (argv[i][j]=='\0')
                  {
                     i++;
                     if (i>=argc)
					 {
						printf("page length not given\n");
						error_found = TRUE;
						break;
					 }
                     if (argv[i][0]!='-')
                     {
                        strcpy(string,argv[i]);
                        j = strlen(argv[i]);
                     }
                     else
					 {
                        j = 0;
						string[0] = '\0';
					 }
                  }
                  else
                  {
                     strcpy(string,&argv[i][j]);
                     j = strlen(argv[i]);
                  }
                  page_length = atoi(string);
                  if ( page_length!=0 && ( page_length<10 || page_width>200 ) )
                  {
                    printf("\nInvalid page length (range 0 or 10-200)\n\n");
					error_found = TRUE;
                  }
                  break;
               case 'd':
                  j++;
                  if (argv[i][j]=='\0')
                  {
                     i++;
                     if (i>=argc)
					 {
						printf("base directory not given\n");
						error_found = TRUE;
						break;
					 }
                     if (argv[i][0]!='-')
                     {
                        strcpy(string,argv[i]);
                        j = strlen(argv[i]);
                     }
					 else
					 {
                        j = 0;
						string[0] = '\0';
					 }
                  }
                  else
                  {
                     strcpy(string,&argv[i][j]);
                     j = strlen(argv[i]);
                  }
                  strcpy(ciocfg_dir,string);
                  if ( ciocfg_dir[ strlen(ciocfg_dir)-1 ] != dir_delim[0] )
                    strcat( ciocfg_dir, dir_delim );
                  break;
               case 'w':
                  j++;
                  if (argv[i][j]=='\0')
                  {
                     i++;
                     if (i>=argc)
					 {
						printf("page width not given\n");
						error_found = TRUE;
						break;
					 }
                     if (argv[i][0]!='-')
                     {
                        strcpy(string,argv[i]);
                        j = strlen(argv[i]);
                     }
					 else
					 {
                        j = 0;
						string[0] = '\0';
					 }
                  }
                  else
                  {
                     strcpy(string,&argv[i][j]);
                     j = strlen(argv[i]);
                  }
                  page_width = atoi(string);
                  if ( page_width<30 || page_width>200 )
                  {
                    printf("\nInvalid page width (range 30-200)\n\n");
					error_found = TRUE;
                  }
                  break;
               default:
                  printf("Unknown option \"%c\"\n",argv[i][j]);
                  j = strlen(argv[i]);
				  error_found = TRUE;
            }
         }
      }
      else
      {
         /* assume it is the compound or block arguments */
         if ( cmpnd[0]=='\0' )
           strcpy(cmpnd, argv[i]);
         else if ( block[0]=='\0' )
           strcpy(block, argv[i]);
         else
		 {
           printf("Unknown argument \"%s\"\n",argv[i]);
		   error_found = TRUE;
		 }
      }
   }

   
   /* exit if there was an error on the command line */
   if ( error_found )
   {
	   exit(0);
   }


   /* if help flag set then print help page and exit */
   if ( help_flag )
   {
     print_title();
     print_help();
     exit(0);
   }


   if ( cmpnd[0]=='\0' )
   {
	   printf("Compound not given\n");
	   error_found = TRUE;
   } else if ( block[0]=='\0' )
   {
	   printf("Block not given\n");
	   error_found = TRUE;
   }


   /* exit if there was an error on the command line */
   if ( error_found )
   {
	   exit(0);
   }


   /* if no flags set then default to ladder logic printout */
   if (!label_flag && !timer_flag && !logic_flag && !xref_flag && !dump_flag)
   {
      label_flag = TRUE;
      logic_flag = TRUE;
      xref_flag = TRUE;
   }

   /* setup left margin */
   lm_spaces[left_margin] = '\0';

   /* load data from source file */
   if (load_source(cmpnd,block)!=OK)
   {
      printf("\nexiting\n");
      exit(1);
   }

   /* process data */
   sscanf(title," %s %s ",ttl_cmpnd,ttl_block);

   /* initialise id info */
   for (i=0; i<=180; i++)
   {
      id_used[i] = 0;
      id_type[i] = 11;  /* normally open contact as default */
   }

   for (row=0; row<num_lines; row++)
   {
      for (col=0; col<8; col++)
      {
         id = c_def[row][col].tech_id;
         if (id==0)
            continue;
         i = id_used[id];
         if ( i>0 )
         {
            if ( id_line[id][i-1]!=row )
               id_line[id][i] = row;
         }
         else
            id_line[id][i] = row;
         id_used[id]++;
         if (col==7)
         {
            c_type = c_def[row][col].c_type;
            switch(c_type)
            {
               case 1:
               case 2:
               case 3:
               case 4:
               case 6:
               case 7:
               case 8:
               case 9:
               case 15:
               case 24:          /* added for Ver 3.2 */
               case 25:
                  id_type[id] = c_type;
                  break;
               case 5:
                  id_type[id] = 4;
                  break;
               case 10:
                  if (id_type[id]==11)
                     id_type[id] = 31;
                  break;
               default:            /* error in coil type */
                  id_type[id] = 31;
            }
         }
      }
   }

   for (id=1; id<MAX_LABELS; id++)
   {
      if ( id_label[id].line1[0]=='\0' && id_label[id].line2[0]=='\0' )
         id_defined[id] = FALSE;
      else
         id_defined[id] = TRUE;
   }

   /* print labels */
   if (label_flag)
      print_labels();

   /* print timers */
   if (timer_flag)
      print_timers();

   /* print ladder logic */
   if (logic_flag)
      print_ladder();

   /* print cross reference lists */
   if (xref_flag)
      print_xref();

   /* print raw logic data */
   if (dump_flag)
      print_dump();

   if ( !first_page && page_length!=0 )
      form_feed();

   return 0;
}


/***************************************************************************
                     load source file and decode
***************************************************************************/

load_source(cmpnd,block)
char *cmpnd, *block;
{
   unsigned short int i;
   char filename[80];
   FILE *src;

   /* convert COMPOUND BLOCK to uppercase */
   for (i=0; i<strlen(cmpnd); i++)
      cmpnd[i] = toupper( cmpnd[i] );

   for (i=0; i<strlen(block); i++)
      block[i] = toupper( block[i] );

   /* construct filename */
   strcpy(filename,ciocfg_dir);
   strcat(filename,cmpnd);
   strcat(filename,dir_delim);
   strcat(filename,block);
   strcat(filename,file_exten);

   /* open source file */
   src = fopen(filename,"rb");
   if (src==NULL)
   {
      printf("unable to open source file for \"%s:%s\"\n",cmpnd,block);
      printf("full filename = %s\n", filename );
      return(ERROR);
   }

   /* read the file header "LAD" */
   if (fread(header,1,4,src)!=4)
   {
      printf("error reading file header\n");
      fclose(src);
      return(ERROR);
   }
   header[4] = '\0';

   if (strcmp(header,"LAD")!=0)
   {
      printf("PLB source file header not \"LAD\"\n");
      fclose(src);
      return(ERROR);
   }

   /* read timer/counter values (preset/reset) */
   if (fread(tc_val, sizeof(struct timer_values), 16, src)!=16)
   {
      printf("error reading timer/counter preset/reset values\n");
      fclose(src);
      return(ERROR);
   }
   for (i=0; i<16; i++)
   {
      tc_val[i].preset = swap_byte( tc_val[i].preset );
      tc_val[i].reset = swap_byte( tc_val[i].reset );
   }

   /* read delimiter */
   if (fread(delimiter, 1, 8, src)!=8)
   {
      printf("error reading delimiter (before logic)\n");
      fclose(src);
      return(ERROR);
   }
   delimiter[8] = '\0';
   if (strcmp(delimiter,delim_string)!=0)
   {
      printf("PLB source file delimiter not \"%s\" (before logic)\n",
         delim_string);
      fclose(src);
      return(ERROR);
   }

   if (fread(&lad_length,sizeof(short int),1,src)!=1)
   {
      printf("error reading ladder definition length\n");
      fclose(src);
      return(ERROR);
   }   
   lad_length = swap_byte(lad_length);

   num_lines = lad_length / 32;

   if (fread(c_def,1,lad_length,src)!=lad_length)
   {
      printf("error reading ladder definition\n");
      fclose(src);
      return(ERROR);
   }   

   /* read delimiter */
   if (fread(delimiter, 1, 8, src)!=8)
   {
      printf("error reading delimiter\n");
      fclose(src);
      return(ERROR);
   }   
   delimiter[8] = '\0';
   if (strcmp(delimiter,delim_string)!=0)
   {
      printf("error reading delimiter (before title)\n");
      fclose(src);
      return(ERROR);
   }

   if (fread(&title_length,sizeof(short int),1,src)!=1)
   {
      printf("error reading title length\n");
      fclose(src);
      return(ERROR);
   }
   title_length = swap_byte( title_length );

   if (fread(title,1,title_length,src)!=title_length)
   {
      printf("error reading title\n");
      fclose(src);
      return(ERROR);
   }

   for(i=0; i<title_length-1; i++)
   {
      if ( title[i]<'0' || title[i]>'z' )
         title[i] = ' ';
   }
   title[i]='\0';

   /* read delimiter */
   if (fread(delimiter, 1, 8, src)!=8)
   {
      printf("error reading delimiter (before labels)\n");
      fclose(src);
      return(ERROR);
   }   
   delimiter[8] = '\0';
   if (strcmp(delimiter,delim_string)!=0)
   {
      printf("PLB source file delimiter not \"%s\"\n",delim_string);
      fclose(src);
      return(ERROR);
   }

   if (fread(&label_length,sizeof(short int),1,src)!=1)
   {
      printf("error reading label length\n");
      fclose(src);
      return(ERROR);
   }
   label_length = swap_byte( label_length );
   num_labels = label_length / 14;

   if (fread(id_label,label_length,1,src)!=1)
   {
      printf("error reading labels\n");
      fclose(src);
      return(ERROR);
   }

   /* read delimiter */
   if (fread(delimiter, 1, 8, src)!=8)
   {
      printf("error reading delimiter (before comments)\n");
      fclose(src);
      return(ERROR);
   }
   delimiter[8] = '\0';
   if (strcmp(delimiter,delim_string)!=0)
   {
      printf("PLB source file delimiter not \"%s\"\n",delim_string);
      fclose(src);
      return(ERROR);
   }

   if (fread(&cmmnt_length,sizeof(short int),1,src)!=1)
   {
      printf("error reading comment length\n");
      fclose(src);
      return(ERROR);
   }
   cmmnt_length = swap_byte( cmmnt_length );

   num_comments = cmmnt_length / 182;
   if (cmmnt_length>0)
   {
      if (fread(comment,cmmnt_length,1,src)!=1)
      {
         printf("error reading comments\n");
         fclose(src);
         return(ERROR);
      }
   }

   fclose(src);

   return(OK);
}


/**************************************************************************
                           output page header
**************************************************************************/

int page_header(cmpnd, block)
char *cmpnd, *block;
{
   int i, spaces, cb_len;

   if ( page_length==0 )
     return(OK);

   if (!first_page)
   {
      putchar('\n');
      form_feed();
   }

   pprintf("FOXBORO I/A PLB BLOCK");

   cb_len = strlen(cmpnd) + strlen(block) + 4;
   spaces = page_width - cb_len - 22;

   for (i=0; i<spaces; i++)
      pprintf(" ");

   pprintf("\"%s:%s\"\n\n",cmpnd,block);

   first_page = FALSE;

   return(OK);
}


/**************************************************************************
                             output form feed
**************************************************************************/

int form_feed()
{
   putchar('\f');
   cur_line = 0;
   cur_col = 0;
   return(OK);
}


/**************************************************************************
                          output tech id labels
**************************************************************************/

int print_labels()
{
   int i, j;
   unsigned int id;
   unsigned char old_class;
   char gap[256], type[10];
   int prt_label[181];
   int num_prt, num_col, col_width, spaces;

   col_width = 35;

   num_prt = 0;

   old_class = 0;

   for (id=1; id<=MAX_LABELS; id++)
   {
      if ( id_defined[id] || id_used[id] || all_label_flag )
      {
         prt_label[num_prt] = id;
         num_prt++;
      }
   }

   num_col = page_width / col_width;
   if (num_col>1)
      spaces = (page_width - (num_col*col_width)) / (2*num_col);
   else
      spaces = ( page_width - col_width) / 2;
   spaces--;

   for (i=0; i<spaces && i<255; i++)
      gap[i] = ' ';
   gap[i] = '\0';

   /* print column titles */
   page_header(ttl_cmpnd, ttl_block);
   pprintf("\n");

   if ( all_label_flag )
     print_centre("LIST OF ALL TECHID LABELS");
   else
     print_centre("LIST OF TECHID LABELS");
   pprintf("\n");

   for (j=0; j<num_col; j++)
      pprintf("%sTechID  Label     Type             %s",gap,gap);
   pprintf("\n");
   for (j=0; j<num_col; j++)
      pprintf("%s------  -----     ----             %s",gap,gap);
   pprintf("\n");


   /* loop thru labels to be printed */
   for (i=0; i<=num_prt; i+=num_col)
   {
      /* loop thru labels in row ( print first line ) */
      for (j=i; j<i+num_col && j<num_prt; j++)
      {
         id = prt_label[j];
         if (id_used[id]!=0)
         {
            strcpy(type,&cntct_name[id_type[id]][1]);
            type[7] = '\0';
         }
         else
            strcpy(type,"NotUsed");
         if ( id_class[id]!=old_class )
         {
            if (j==i)
               pprintf("\n\n");
            else
               break;
         }
         strncpy(line1,id_label[id].line1,7);
         line1[7] = '\0';
         pprintf("%s%-6s  %-7s  %-7s           %s",gap,id_name[id],line1,type,gap);
         old_class = id_class[id];
      }
      pprintf("\n");

      /* loop thru labels in row ( print second line ) */
      for (j=i; j<i+num_col && j<num_prt; j++)
      {
         id = prt_label[j];
         if ( id_class[id]!=old_class )
         {
            if (i!=j)
            {
               i = j - num_col;
               break;
            }
         }
         strncpy(line2,id_label[id].line2,7);
         line2[7] = '\0';
         pprintf("%s        %-7s                    %s", gap, line2, gap);
         old_class = id_class[id];
      }
      pprintf("\n\n");

      /* if not last row of labels */
      if ( i < (num_prt-num_col) )
      {
        /* if at bottom then output footer and header */
        if (page_length!=0 && cur_line>(page_length-6))
        {
           /* print column titles */
           page_header(ttl_cmpnd, ttl_block);
           pprintf("\n");
           for (j=0; j<num_col; j++)
             pprintf("%sTechID  Label     Type             %s",gap,gap);
           pprintf("\n");
           for (j=0; j<num_col; j++)
              pprintf("%s------  -----     ----             %s",gap,gap);
           pprintf("\n");
        }
      }

      /* next row of labels */
   }

   return(OK);
}


/**************************************************************************
                            output ladder logic
**************************************************************************/

int print_ladder()
{
   unsigned int i;
   unsigned int row, col;
   unsigned char connect[8], c_type[8], tech_id[8];

   page_header(ttl_cmpnd, ttl_block);
   pprintf("\n");
   print_centre("LADDER LOGIC DIAGRAM");
   pprintf("\n");

   for (row=0; row<num_lines && c_def[row][0].connect != 0x7C; row++)
   {
      /* find contact definition */
      for (col=0; col<8; col++)
      {
         connect[col] = c_def[row][col].connect;
         c_type[col] = c_def[row][col].c_type;
         tech_id[col] = c_def[row][col].tech_id;
      }

      /* line 1 */
      pprintf("   ");
      for (col=0; col<8; col++)
      {
         if ( col==0 && connect[col]==0xf0 )
            pprintf("\n\n");
         if ( connect[col] & 1 )
            pprintf("|        ");
         else
            pprintf("         ");
      }
      pprintf("\n");

      /* line 2 */
      pprintf("   ");
      for (col=0; col<8; col++)
      {
         if ( connect[col] & 1 )
            pprintf("| ");
         else
            pprintf("  ");
         pprintf("%-6s ",id_name[ tech_id[col] ] );
      }
      pprintf("\n");

      /* line 3 */
      pprintf("%-3d",row+1);
      for (col=0; col<8; col++)
      {
         if ( connect[col]==0 || (connect[col] & 0x0c) )
         {
            pprintf("         ");
            continue;
         }
         else
         {
            if ( connect[col] & 3 )
               pprintf("+");
            else
               pprintf("-");
   
            if ((connect[col]==0x93 || connect[col]==0xa1 || connect[col]==0xe3 )
                  && c_type[col]==0 )
               pprintf("        ",connect[col]);
            else
               pprintf("%s",&cntct_name[c_type[col]][1]);
         }
      }

      /* if timer/count then print preset value */
      if ( page_width >= 84 )
      {
         switch (c_type[7])
         {
            case 6:
            case 7:
               i = ( tech_id[7] - TC_OFFSET ) / 2;
               pprintf("%6.1fP",tc_val[i].preset * 0.1 );
               break;
            case 8:
            case 9:
               i = ( tech_id[7] - TC_OFFSET ) / 2;
               pprintf("%6dP",tc_val[i].preset );
               break;
            case 24:
            case 25:
               i = ( tech_id[7] - TC_OFFSET ) / 2;
               pprintf("%6.1fP",tc_val[i].preset * 0.1 );
               break;
         }
      }
      pprintf("\n");
   

      /* line 4 */
      pprintf("   ");
      for (col=0; col<8; col++)
      {
         if ( connect[col] & 2 )
            pprintf("|");
         else
            pprintf(" ");

         if (tech_id[col]!=0)
         {
            strcpy( line1, id_label[ tech_id[col] ].line1);
            line1[7] = '\0';
         }
         else
            strcpy( line1, "       ");

         pprintf(" %-7s",line1);
      }

      /* if timer/count then print preset value */
      if (page_width>83)
      {
         switch (c_type[7])
         {
            case 6:
            case 7:
               i = ( tech_id[7] - TC_OFFSET ) / 2;
               pprintf("%6.1fR",tc_val[i].reset * 0.1 );
               break;
            case 8:
            case 9:
               i = ( tech_id[7] - TC_OFFSET ) / 2;
               pprintf("%6dR",tc_val[i].reset );
               break;
            case 24:
            case 25:
               i = ( tech_id[7] - TC_OFFSET ) / 2;
               pprintf("%6.1fR",tc_val[i].reset * 0.1 );
               break;
         }
      }
      pprintf("\n");

      /* line 5 */
      pprintf("   ");
      for (col=0; col<8; col++)
      {
         if ( connect[col] & 2 )
            pprintf("|");
         else
            pprintf(" ");

         if (tech_id[col]!=0)
         {
            strcpy( line2, id_label[ tech_id[col] ].line2);
            line2[7] = '\0';
         }
         else
            strcpy( line2, "       ");

         pprintf(" %-7s",line2);
      }
      pprintf("\n");

      /* if at bottom then output footer and header */
      if ( page_length!=0 && cur_line>(page_length-6))
         page_header(ttl_cmpnd, ttl_block);

      /* find and print comment */
      for (i=0; i<num_comments; i++)
      {
         if (comment[i].position==row)
         {
            strncpy(line1,comment[i].line1,60);
            strncpy(line2,comment[i].line2,60);
            strncpy(line3,comment[i].line3,60);
            line1[60] = '\0';
            line2[60] = '\0';
            line3[60] = '\0';
            pprintf("\n            %s\n",line1);
            pprintf("            %s\n",line2);
            pprintf("            %s\n",line3);
            pprintf("\n");
         }
      }

      /* if not end of ladder logic */
      if ( row < (num_lines-1) && c_def[row+1][0].connect != 0x7C )
      {
         /* if at bottom then output footer and header */
         if (page_length!=0 && cur_line>(page_length-6))
            page_header(ttl_cmpnd, ttl_block);
      }

   }
   pprintf("\n\n");

   return(OK);
}


/**************************************************************************
                      output cross reference lists
**************************************************************************/

int print_xref()
{
   int id, i, j;
   int num_col;

   num_col = ( page_width - 17 ) / 6;

   page_header(ttl_cmpnd, ttl_block);
   pprintf("\n");
   print_centre("CROSS REFERENCE LISTING OF TECHIDS");
   pprintf("\n");

   for (id=1; id<(int)num_labels; id++)
   {
      if ( id_used[id]==0 )
         continue;

      strcpy(line1,id_label[id].line1);
      line1[7] = '\0';
      strcpy(line2,id_label[id].line2);
      line2[7] = '\0';

      for (i=0; i<id_used[id]; i+= num_col )
      {
         for (j=i; j<i+num_col && j<id_used[id]; j++)
         {
            if (j==0)
               pprintf("%-6s  %-7s",id_name[id],line1);
            else
            {
               if (j==num_col)
                  pprintf("        %-7s",line2);
            }

            pprintf("  %3d",id_line[id][j]+1);
   
            if (c_def[id_line[id][j]][7].tech_id==id)
               pprintf("*");
            else
               pprintf(" ");

         /* next entry */
         }
         pprintf("\n");

      /* next line */
      }
      if (id_used[id]<=num_col)
         pprintf("        %-7s\n",line2);

      pprintf("\n");

      /* if not last label */
      if ( id < (int)(num_labels-1) && id_used[id+1]!=0 )
      {
         /* if at bottom then output footer and header */
         if (page_length!=0 && cur_line>(page_length-6))
            page_header(ttl_cmpnd, ttl_block);
      }

   /* next id */
   }

   return(OK);
}



/**************************************************************************
                           output timer values
**************************************************************************/

int print_timers()
{
   int i, id;
   float preset, reset;

   page_header(ttl_cmpnd, ttl_block);
   pprintf("\n");
   if ( all_timer_flag )
     print_centre("LIST OF ALL TIMER/COUNTER PRESET AND RESET VALUES");
   else
     print_centre("LIST OF TIMER/COUNTER PRESET AND RESET VALUES");
   pprintf("\n");

   for (i=0; i<16; i++)
   {
      id = i*2 + TC_OFFSET;
      if (id_used[id]!=0 || all_timer_flag)
      {
        switch ( id_type[id] )
        {
          case 6:     /* --(RTO)-- */
          case 7:     /* --(RTF)-- */
            preset = (float)tc_val[i].preset * (float)0.1;
            reset  = (float)tc_val[i].reset * (float)0.1;
            break;
          case 8:     /* --(CTU)-- */
          case 9:     /* --(CTD)-- */
            preset = (float)tc_val[i].preset;
            reset  = (float)tc_val[i].reset;
          case 24:    /* --(TON)-- */
          case 25:    /* --(TOF)-- */
            preset = (float)tc_val[i].preset * (float)0.1;
            reset  = (float)tc_val[i].reset * (float)0.1;
            break;
          default:
            preset = 0.0;
            reset = 0.0;
        }
        if ( id_used[id]!=0 )
          pprintf("%s      preset  %7.1f       reset  %7.1f      %s\n",
            id_name[id], preset, reset, cntct_name[id_type[id]] );
        else
          pprintf("%s      preset  %7.1f       reset  %7.1f      %s\n",
            id_name[id], preset, reset, "" );
      }
   }
   pprintf("\n");

   return(OK);
}


/***************************************************************************
                       output hex dump of ladder logic
***************************************************************************/

int print_dump()
{
   unsigned int i, j;

   page_header(ttl_cmpnd, ttl_block);

   printf("Raw Hexadecimal Dump from Ladder Logic\n");

   for (i=0; i<num_lines; i++)
   {
      pprintf("%02X - ",i);
      for (j=0; j<8; j++)
         pprintf("%02X    ",c_def[i][j].connect);
      pprintf("\n     ");
      for (j=0; j<8; j++)
         pprintf("%02X    ",c_def[i][j].c_type);
      pprintf("\n     ");
      for (j=0; j<8; j++)
         pprintf("%02X    ",c_def[i][j].tech_id);
      pprintf("\n     ");
      for (j=0; j<8; j++)
         pprintf("%02X    ",c_def[i][j].dummy);
      pprintf("\n\n");
   }
   pprintf("\n");

   return(OK);
}


/***************************************************************************
                               format output
***************************************************************************/

int pprintf(va_alist)
va_dcl
{
   char *fmt;
   char buffer[256];
   char *x;
   unsigned int i;
   va_list arg_ptr;

   va_start(arg_ptr);

   fmt = va_arg(arg_ptr, char*);
   vsprintf(buffer,fmt,arg_ptr);
   va_end(arg_ptr);

   for (i=0, x=buffer; i<strlen(buffer); i++, x++ )
   {
      switch (*x)
      {
         case '\f':
            page_num++;
            cur_line = 0;
            putchar(*x);
            printf("%s",lm_spaces);
            cur_col = left_margin;
            break;
         case '\n':
            putchar(*x);
            printf("%s",lm_spaces);
            cur_col = left_margin;
            cur_line++;
            break;
         default:
            cur_col++;
            putchar(*x);
      }   
   }
   
   return(OK);
}


/***************************************************************************
                 swap high and low bytes of 16 bit integer
***************************************************************************/
/* conditional compile depending of whether byte swap it needed           */
/* swap for SPARC processors but NOT for Intel x86 processors             */

short int swap_byte( src )
short int src;
{
#ifdef _M_IX86
   return (src);
#else
   short int lo, hi;
   short int rtn;

   lo = src & 0x00ff;
   hi = ( src & 0xff00 ) >> 8;
   rtn = ( lo << 8 ) | hi;

   return(rtn);
#endif
}


/***************************************************************************
                         decode an encrypted string
***************************************************************************/

int print_hex( src, dest )
char *src, *dest;
{
   int i;
   int len;

   len = strlen(src);

   for (i=0; i<len; i++)
   {
      *dest = *src ^ 0x05;
      src++;
      dest++;
   }
   *dest = '\0';

   return 0;
}


/***************************************************************************
                                print title
***************************************************************************/

void print_title()
{
  printf("\n");
  printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
  printf("\n");
  printf("                 P L B   B L O C K   S O U R C E   L I S T E R\n");
  printf("                            Version 1.2 - May 1999\n");
  printf("\n");
  printf("                      by Kevin Millar (kmillar@ihug.co.nz)\n");
  printf("                         W. Arthur Fisher (NZ) Limited\n");
  printf("\n");
  printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
  printf("\n");
}


/***************************************************************************
                               print summary
***************************************************************************/

void print_summary()
{
  printf("usage : plb_dump [options] CMPND BLOCK\n" );
  printf("\n");
  printf("options -help       print help pages\n");
  printf("        -l[a]       print labels (a = all)\n");
  printf("        -t[a]       print timer/counter values (a = all)\n");
  printf("        -r          print ladder logic rungs\n");
  printf("        -x          print cross-reference listing\n");
  printf("        -d base_dir set name of base directory\n");
  printf("        -p length   set page length (in lines)\n");
  printf("        -w width    set page width (in characters)\n");
  printf("\n");
  printf("default options are -l -r -x\n");
  printf("\n");
}


/***************************************************************************
                               print help page
***************************************************************************/

void print_help()
{

  printf("This utility reads the ladder logic definition file for an I/A PLB block,\n");
  printf("and prints the contents of the file to the standard output device.  The\n");
  printf("output can include a list of the labels used (or all labels in the block),\n");
  printf("the ladder logic of the block, a cross-reference list of the labels used\n");
  printf("and timer/counter preset and reset values.\n");
  printf("\n");
  printf("The ladder logic definition files are binary files stored in the compound\n");
  printf("directory, named after the block and with a .p extension.  Typcially, the\n");
  printf("compound directories are located under the Control Configurator directory.\n");
  printf("By default, the utility will assume the compound directories are located\n");
  printf("in the directory \"/opt/fox/ciocfg\".  This is the standard location for\n");
  printf("stations hosted by a 51 Series processor.\n");
  printf("\n");
  printf("Options\n");
  printf("\n");
  printf("-help       prints the help pages\n");
  printf("\n");
  printf("                              Output Options\n");
  printf("\n");
  printf("-l[a]       prints the labels (and types) of the TechIDs used within the\n");
  printf("            PLB block.  The additional 'a' option will force the utility\n");
  printf("            to output the labels for ALL of the TechIDs.  The inputs to\n");
  printf("            the PLB block (contacts and flags) are shown as normally open\n");
  printf("            contacts.\n");
  printf("\n");
  printf("-t[a]       prints a list of the preset and reset values defined for the\n");
  printf("            timers and counters.  The additional 'a' option will force the\n");
  printf("            utility to output the labels for ALL of the timers and counters.\n");
  printf("\n");
  printf("-r          prints ladder logic rungs\n");
  printf("\n");
  printf("-x          print cross-reference listing of the labels used within the\n");
  printf("            ladder logic.  Each TechID used is output with its label and a\n");
  printf("            list of rungs that the TechID appears in.  If the rung outputs\n");
  printf("            to the TechID, the rung number is marked with a '*'.\n");
  printf("\n");
  printf("                              Location Option\n");
  printf("\n");
  printf("-d base_dir define the name of base directory.  By default the utility\n");
  printf("            assumes the compound directories are located in the directory\n");
  printf("            \"/opt/fox/ciocfg/\".  This option will allow the utility to load\n");
  printf("            the ladder logic definition files from other locations.\n");
  printf("\n");
  printf("            An example of this would be able to access the files from a SaveAll\n");
  printf("            floppy.  If the SaveAll floppy is mounted on the directory \"f0\",\n");
  printf("            use the option \"-d /f0\" to access the ladder logic files.  Use the\n");
  printf("            option \"-d .\" when the compound directories are located in the\n");
  printf("            current directory\n");
  printf("\n");
  printf("                              Format Options\n");
  printf("\n");
  printf("-p length   set page length (in lines).  The default length is 60 lines.\n");
  printf("            For a continuous output, use a length of 0.\n");
  printf("\n");
  printf("-w width    set page width (in characters).  The default width is 84\n");
  printf("            characters.  If a narrower width is selected then the timer\n");
  printf("            and counter preset and reset values can not be printed beside\n");
  printf("            the timer or counter in the ladder logic rungs.\n");
  printf("\n");
  printf("If none of the output options are specified, the utility will output the\n");
  printf("labels, the ladder logic rungs and cross reference listings.\n");
  printf("\n");
  printf("Please email comments, bug reports etc to \"kmillar@ihug.co.nz\".\n");
  printf("\n");
}


int print_centre( char *string )
{
  int i, gap;

  gap = ( page_width - strlen(string) ) / 2;

  if ( gap < 0 )
    gap = 0;

  for ( i=0; i<gap ; i++)
    pprintf(" ");

  pprintf("%s\n", string);

  return OK;
}
