/* definition data types and initialized arrays */

#define ERROR 0
#define OK    !ERROR
#define FALSE 0
#define TRUE  !FALSE

#define DEF_PAGE_LENGTH	60
#define DEF_PAGE_WIDTH	84

#define MAX_LABELS 180

/* file structure and directories for 50 series */
char ciocfg_dir[128] = "/opt/fox/ciocfg/";
char dir_delim[] = "/";
char file_exten[] = ".p";

struct timer_values {
   unsigned short int preset;
   unsigned short int reset;
};

struct contact_def {
   unsigned char connect;
   unsigned char c_type;
   unsigned char tech_id;
   unsigned char dummy;
};

struct tech_id_label {
   char line1[7];
   char line2[7];
};

struct comment_entry {
   char line1[60];
   char line2[60];
   char line3[60];
   unsigned char position;
   unsigned char dummy;
};

#define TC_OFFSET     1
#define INIT_OFFSET   33
#define FAILS_OFFSET  34
#define COMMF_OFFSET  35
#define POWERF_OFFSET 36
#define CIN_OFFSET    37
#define CO_OFFSET     69
#define IFL_OFFSET    85
#define OFL_OFFSET    117
#define INT_OFFSET    149

#define COIL_TYPE     0
#define TIMER_TYPE    1
#define COUNT_TYPE    2

char id_name[181][8] = { "      ",
                         "TC01_S", "TC01_O", "TC02_S", "TC02_O",
                         "TC03_S", "TC03_O", "TC04_S", "TC04_O",
                         "TC05_S", "TC05_O", "TC06_S", "TC06_O",
                         "TC07_S", "TC07_O", "TC08_S", "TC08_O",
                         "TC09_S", "TC09_O", "TC10_S", "TC10_O",
                         "TC11_S", "TC11_O", "TC12_S", "TC12_O",
                         "TC13_S", "TC13_O", "TC14_S", "TC14_O",
                         "TC15_S", "TC15_O", "TC16_S", "TC16_O",
                         "INIT",   "FAILS",  "COMMF",  "POWERF",
                         "CIN_1",  "CIN_2",  "CIN_3",  "CIN_4",
                         "CIN_5",  "CIN_6",  "CIN_7",  "CIN_8",
                         "CIN_9",  "CIN_10", "CIN_11", "CIN_12",
                         "CIN_13", "CIN_14", "CIN_15", "CIN_16",
                         "CIN_17", "CIN_18", "CIN_19", "CIN_20",
                         "CIN_21", "CIN_22", "CIN_23", "CIN_24",
                         "CIN_25", "CIN_26", "CIN_27", "CIN_28",
                         "CIN_29", "CIN_30", "CIN_31", "CIN_32",
                         "CO_1",   "CO_2",   "CO_3",   "CO_4",
                         "CO_5",   "CO_6",   "CO_7",   "CO_8",
                         "CO_9",   "CO_10",  "CO_11",  "CO_12",
                         "CO_13",  "CO_14",  "CO_15",  "CO_16",
                         "IFL_1",  "IFL_2",  "IFL_3",  "IFL_4",
                         "IFL_5",  "IFL_6",  "IFL_7",  "IFL_8",
                         "IFL_9",  "IFL_10", "IFL_11", "IFL_12",
                         "IFL_13", "IFL_14", "IFL_15", "IFL_16",
                         "IFL_17", "IFL_18", "IFL_19", "IFL_20",
                         "IFL_21", "IFL_22", "IFL_23", "IFL_24",
                         "IFL_25", "IFL_26", "IFL_27", "IFL_28",
                         "IFL_29", "IFL_30", "IFL_31", "IFL_32",
                         "OFL_1",  "OFL_2",  "OFL_3",  "OFL_4",
                         "OFL_5",  "OFL_6",  "OFL_7",  "OFL_8",
                         "OFL_9",  "OFL_10", "OFL_11", "OFL_12",
                         "OFL_13", "OFL_14", "OFL_15", "OFL_16",
                         "OFL_17", "OFL_18", "OFL_19", "OFL_20",
                         "OFL_21", "OFL_22", "OFL_23", "OFL_24",
                         "OFL_25", "OFL_26", "OFL_27", "OFL_28",
                         "OFL_29", "OFL_30", "OFL_31", "OFL_32",
                         "INT_01", "INT_02", "INT_03", "INT_04",
                         "INT_05", "INT_06", "INT_07", "INT_08",
                         "INT_09", "INT_10", "INT_11", "INT_12",
                         "INT_13", "INT_14", "INT_15", "INT_16",
                         "INT_17", "INT_18", "INT_19", "INT_20",
                         "INT_21", "INT_22", "INT_23", "INT_24",
                         "INT_25", "INT_26", "INT_27", "INT_28",
                         "INT_29", "INT_30", "INT_31", "INT_32" };

unsigned char id_class[181] = { 0,
            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,   /* TCxx  */
            1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
            2, 2, 2, 2,                                       /* MISC  */
            3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,   /* CINxx */
            3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,   
            4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,   /* COxx  */
            5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,   /* IFLxx */
            5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
            6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,   /* OFLxx */
            6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
            7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,   /* INTxx */
            7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7 };


char cntct_name[32][10] ={ "---------",    /*  0  */
                           "--(MCR)--",    /*  1  */
                           "--(ZCL)--",    /*  2  */
                           "---( )---",    /*  3  */
                           "---(L)---",    /*  4  */
                           "---(U)---",    /*  5  */
                           "--(RTO)--",    /*  6  */
                           "--(RTF)--",    /*  7  */
                           "--(CTU)--",    /*  8  */
                           "--(CTD)--",    /*  9  */
                           "--(RST)--",    /*  10 */
                           "---| |---",    /*  11 */
                           "---|\\|---",   /*  12 */
                           "   0x0D  ",    /*  13 */
                           "   0x0E  ",    /*  14 */
                           "---(\\)---",   /*  15 */
                           "   0x10  ",    /*  16 */
                           "   0x11  ",    /*  17 */
                           "   0x12  ",    /*  18 */
                           "   0x13  ",    /*  19 */
                           "   0x14  ",    /*  20 */
                           "   0x15  ",    /*  21 */
                           "--(NCL)--",    /*  22 */
                           "--(NCR)--",    /*  23 */
                           "--(TON)--",    /*  24 */
                           "--(TOF)--",    /*  25 */
                           "   0x1A  ",    /*  26 */
                           "   0x1B  ",    /*  27 */
                           "   0x1C  ",    /*  28 */
                           "   0x1D  ",    /*  29 */
                           " - - - - ",    /*  30 */
                           " *ERROR* "     /*  31 */  };

char delim_string[] = "==^==^==";

struct timer_values  tc_val[16];
struct contact_def   c_def[100][8];
struct tech_id_label id_label[181];
unsigned char id_used[181];
unsigned char id_line[181][100];
unsigned char id_defined[181];
unsigned char id_type[181];
struct comment_entry comment[100];
char   title[256];
char   line1[256];
char   line2[256];
char   line3[256];
char   ttl_cmpnd[20];
char   ttl_block[20];
int    cur_line = 0;
int    cur_col = 0;
int    page_num = 0;
int    left_margin = 0;
int    page_width = DEF_PAGE_WIDTH;
int    page_length = DEF_PAGE_LENGTH;
char   lm_spaces[] = "                                        ";

char header[5];
char delimiter[9];

unsigned short int lad_length;
unsigned short int title_length;
unsigned short int label_length;
unsigned short int cmmnt_length;

unsigned int num_lines;
unsigned int num_labels;
unsigned int num_comments;

char first_page = TRUE;
char label_flag = FALSE;
char all_label_flag = FALSE;
char timer_flag = FALSE;
char help_flag = FALSE;
char all_timer_flag = FALSE;
char logic_flag = FALSE;
char xref_flag = FALSE;
char dump_flag = FALSE;
char error_found = FALSE;

short int swap_byte();

